<#ibiztemplate>
TARGET=PSSYSTEM
</#ibiztemplate>
<#if sys.getAllPSWorkflows()?? && srflist(sys.getAllPSWorkflows())?size gt 0>
package ${pub.getPKGCodeName()}.job;

import feign.Client;
import feign.Contract;
import feign.Feign;
import feign.codec.Decoder;
import feign.codec.Encoder;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;
import org.springframework.cloud.openfeign.ribbon.LoadBalancerFeignClient;

import java.util.List;
import java.util.ArrayList;

import ${pub.getPKGCodeName()}.util.config.SysInfoProperties;
import ${pub.getPKGCodeName()}.util.feign.FeignRequestInterceptor;
import ${pub.getPKGCodeName()}.util.config.UniWFClientProperties;
import ${pub.getPKGCodeName()}.util.feign.UniWFFeignClient;
import ${pub.getPKGCodeName()}.util.domain.PSDEWF;
import ${pub.getPKGCodeName()}.util.domain.PSDataEntity;
import ${pub.getPKGCodeName()}.util.domain.PSSystem;
import ${pub.getPKGCodeName()}.util.domain.PSWorkflow;
import ${pub.getPKGCodeName()}.util.domain.WFEngine;
import ${pub.getPKGCodeName()}.util.domain.WFMode;
import ${pub.getPKGCodeName()}.util.domain.Workflow;

/**
 *
 */
@Component
public class WFJob implements ApplicationRunner {

    UniWFFeignClient uniWFFeignClient;

    @Autowired
    SysInfoProperties sysInfoProperties;

    @Autowired
    public WFJob(Decoder decoder, Encoder encoder, Client client, Contract contract, FeignRequestInterceptor feignRequestInterceptor, UniWFClientProperties uniWFClientProperties) {
        if (uniWFClientProperties.getServiceId()!=null) {
    		Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(encoder)
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.uniWFFeignClient = nameBuilder.target(UniWFFeignClient.class,"http://" + uniWFClientProperties.getServiceId() + "/") ;
		} else if (uniWFClientProperties.getServiceUrl()!=null) {
			if (client instanceof LoadBalancerFeignClient) {
				client = ((LoadBalancerFeignClient) client).getDelegate();
			}
			Feign.Builder nameBuilder = Feign.builder()
                    .client(client)
                    .encoder(encoder)
                    .decoder(decoder)
                    .contract(contract)
                    .requestInterceptor(feignRequestInterceptor)
                    ;
        	this.uniWFFeignClient = nameBuilder.target(UniWFFeignClient.class, "http://" + uniWFClientProperties.getServiceUrl() + "/") ;
		}
    }

    @Override
    public void run(ApplicationArguments args) throws Exception {
        // 注册统一工作流（域注册）
        Workflow workflow = new Workflow();
        workflow.setWorkflowid(sysInfoProperties.getDomainid());
        workflow.setWorkflowname(sysInfoProperties.getDomainname());
        workflow.setLogicname(sysInfoProperties.getDomainlogicname());
        uniWFFeignClient.registDomain(workflow);

        // 注册工作流引擎
        List<WFEngine> wfengines = new ArrayList<WFEngine>();
        WFEngine wfengine1 = new WFEngine();
        wfengine1.setWfengineid("ACTIVITI");
        wfengine1.setWfenginename("ACTIVITI");
        wfengine1.setWfenginetype("ACTIVITI");
        wfengine1.setLogicname("ACTIVITI");
        wfengines.add(wfengine1);
        uniWFFeignClient.registWFEngine(wfengines);

        // 注册业务系统（部署系统）
        PSSystem pssystem = new PSSystem();
        pssystem.setPssystemid(sysInfoProperties.getSystemid());
        pssystem.setPssystemName(sysInfoProperties.getSystemname());
        pssystem.setWorkflowid(sysInfoProperties.getDomainid());
        pssystem.setServiceid(sysInfoProperties.getServiceid());
        pssystem.setServiceurl(sysInfoProperties.getServiceurl());
        pssystem.setLogicname(sysInfoProperties.getSystemlogicname());
        uniWFFeignClient.registPSSystem(pssystem);

        // 注册业务工作流
        List<PSWorkflow> psworkflows = new ArrayList<PSWorkflow>();
    <#assign i = 0>
    <#list sys.getAllPSWorkflows() as pswf >
        PSWorkflow psworkflow${i} = new PSWorkflow();
        psworkflow${i}.setPsworkflowid("${pswf.getId()}");
        psworkflow${i}.setPsworkflowname("${pswf.getCodeName()}");
        psworkflow${i}.setLogicname("${pswf.getLogicName()}");
        psworkflow${i}.setWorkflowid(sysInfoProperties.getDomainid());
        psworkflow${i}.setPssystemid(sysInfoProperties.getSystemid());
        psworkflow${i}.setWfenginetype("ACTIVITI");
        psworkflows.add(psworkflow${i});
        <#assign i++>
    </#list>
        uniWFFeignClient.registPSWorkflow(psworkflows);

        // 注册业务实体
        List<PSDataEntity> psdataentities = new ArrayList<PSDataEntity>();
    <#assign i = 0>
    <#if sys.getAllPSDataEntities()??>
        <#list sys.getAllPSDataEntities() as psde >
            <#if psde.hasPSDEWF()>
        PSDataEntity psdataentity${i} = new PSDataEntity();
        psdataentity${i}.setPsdataentityid("${psde.getId()}");
        psdataentity${i}.setPsdataentityname("${psde.getCodeName()}");
        psdataentity${i}.setLogicname("${psde.getLogicName()}");
        psdataentity${i}.setWorkflowid(sysInfoProperties.getDomainid());
        psdataentity${i}.setPssystemid(sysInfoProperties.getSystemid());
        psdataentities.add(psdataentity${i});
            <#assign i++>
            </#if>
        </#list>
    </#if>
        uniWFFeignClient.registPSDataEntity(psdataentities);

        // 注册业务实体工作流
        List<PSDEWF> psdewfs = new ArrayList<PSDEWF>();
    <#assign i = 0>
    <#list sys.getAllPSWorkflows() as pswf >
        <#if pswf.getPSWFDEs()??>
            <#assign deArr = [] />
            <#list pswf.getPSWFDEs() as pswfde>
                <#if !deArr?seq_contains(pswfde.getId())>
        PSDEWF psdewf${i} = new PSDEWF();
        psdewf${i}.setPsdewfid("${pswfde.getId()}");
        psdewf${i}.setPsdewfname("${pswfde.getName()}");
        psdewf${i}.setLogicname("${pswfde.getPSDataEntity().getLogicName()}");
        psdewf${i}.setWfproxymode(${pswfde.getWFProxyMode()});
        psdewf${i}.setPsdataentityid("${pswfde.getPSDataEntity().getId()}");
        psdewf${i}.setWorkflowid(sysInfoProperties.getDomainid());
        psdewf${i}.setPssystemid(sysInfoProperties.getSystemid());
        psdewfs.add(psdewf${i});
                    <#assign deArr = deArr + [pswfde.getId()] >
                    <#assign i++>
                </#if>
            </#list>
        </#if>
    </#list>
        uniWFFeignClient.registPSDEWF(psdewfs);

        // 注册业务工作流模式
<#--        List<WFMode> wfmodes = new ArrayList<WFMode>();-->
<#--        模型还没找到-->
<#--    <#assign i = 0>-->
<#--    <#if sys.getAllPSDataEntities()??>-->
<#--        <#list sys.getAllPSWFMode() as pswf >-->
<#--        WFMode wfmode${i} = new WFMode();-->
<#--        wfmode${i}.setPSWorkflowid("${pswf.getId()}");-->
<#--        wfmode${i}.setPSWorkflowname("${pswf.getCodeName()}");-->
<#--        wfmode${i}.setLogicname("${pswf.getLogicName()}");-->
<#--        wfmodes.add(wfmode${i});-->
<#--        <#assign i++>-->
<#--        </#list>-->
<#--    </#if>-->
<#--        uniWFFeignClient.registWFMode(wfmodes);-->
    }
}
</#if>
